/*******************************************************************************
 *
 * Copyright (c) 2016-2026 HYCON Technology, Inc.
 * All rights reserved.
 * HYCON Technology <www.hycontek.com>
 *
 * HYCON reserves the right to amend this code without notice at any time.
 * HYCON assumes no responsibility for any errors appeared in the code,
 * and HYCON disclaims any express or implied warranty, relating to sale
 * and/or use of this code including liability or warranties relating
 * to fitness for a particular purpose, or infringement of any patent,
 * copyright or other intellectual property right.
 *
 * -----------------------------------------------------------------------------
 * File Name    : user_DEF.h
 * IDE tooling  : AndeSight C/C++ IDE, version: 2.0.1 Build ID : 201303201736
 * Device Ver.  : V0.9 crt0.o for HY16F19xx & HY16F18xx MCU.
 * Library Ver. : 1.1
 * MCU Device   : HY16F198B-L100 @3.0V
 * Description  : This file implements the customer's function.
 * Created Date : 2016/05/28
 * Created by   :
 *
 ******************************************************************************/

#ifndef _USER_DEF_H_
#define _USER_DEF_H_

/*----------------------------------------------------------------------------*/
/* Header file include                                                        */
/*----------------------------------------------------------------------------*/
#include "DrvREG32.h"
#include "SpecialMacro.h"
//#include "stdint.h"

/*----------------------------------------------------------------------------*/
/* Structure definition                                                       */
/*----------------------------------------------------------------------------*/
volatile typedef union _USERFLAG0
{
    char _byte;
    struct
    {
        unsigned b_IRQ128_Flag:1;                       //RTC IRQ period time 128Hz enable[1]/disable[0] flag.
        unsigned b_KInPro_Flag:1;
        unsigned b_HAO_EI_Flag:1;
        unsigned b_ALMInt_Flag:1;
        unsigned b_RTCInt_Flag:1;
        unsigned b_AlmEI_Flag:1;                        //Alarm function enable[1]/disable[0] flag.
        unsigned b_DspGo_Flag:1;
        unsigned b_bit7:1;
    };
} USERFLAG0;

volatile typedef union _USERFLAG1
{
    char _byte;
    struct
    {
        unsigned b_StpWGo_Flag:1;                       //Sart[1]/Stop[0] status flag.
        unsigned b_QAdjGo_Flag:1;                       //Quick adjust go status flag.
        unsigned b_AlmGo_Flag:1;                        //Alarm going status flag.
        unsigned b_KScan_Flag:1;                        //Key scaning enable[1]/disable[0] flag.
        unsigned b_SetEI_Flag:1;                        //Time setup enable flag.
        unsigned b_KeyOn_Flag:1;
        unsigned b_KeyInt_Flag:1;                       //PT1.1 & PT1.2 int. pending flag status.
        unsigned b_bit7:1;
    };
} USERFLAG1;

/*----------------------------------------------------------------------------*/
/* Global variable definition                                                 */
/*----------------------------------------------------------------------------*/
USERFLAG0 UserFlag0;
USERFLAG1 UserFlag1;
//extern unsigned char DisplayBuffer[18];
extern unsigned int  Sleep_Time_Cnt;
extern unsigned char Beep_Tone_Cnt;
extern unsigned int  STW_100;

/*----------------------------------------------------------------------------*/
/* Constant value definition                                                  */
/*----------------------------------------------------------------------------*/
#define BIT0                    0x01                    //
#define BIT1                    0x02
#define BIT2                    0x04
#define BIT3                    0x08
#define BIT4                    0x10
#define BIT5                    0x20
#define BIT6                    0x40
#define BIT7                    0x80

#define INIT_GO_TIME            (10000000u/78125u)      //Time count= (1Sec)/(7.8125mS time base).
#define PWM_4KHZ_TIME           (2000000u/4000u)        //Time count= HS_CK/4KHz= 2MHz/4KHz.
#define PWM_2KHZ_TIME           (2000000u/2000u)        //Time count= HS_CK/2KHz= 2MHz/2KHz.
#define BEEP_TONE_TIME          (80u/8u)                //Time count= (beep time)/(8mS time base).
#define NO_KEY_RESET_TIME       (1000u/32u)             //Time count= (1000mS)/(32mS time base).
#define ALM_TONE_TIME           (60*4)                  //4beep tone per second, so 1 minute has 60*4= 240 times beep tone.
#define SET_RELEASE_TIME        (300000000u/78125u)     //Time count= (30Sec)/(7.8125mS time base).
#define KEY_SET_OFF_TIME        (20000000u/78125u)      //Time count= (2Sec)/(7.8125mS time base).
#define QADJ_TIME_1HZ           (10000000u/78125u)      //Time count= (1Sec)/(7.8125mS time base).
#define QADJ_TIME_4HZ           (2500000u/78125u)       //Time count= (250mS)/(7.8125mS time base).
#define QADJ_TIME_8HZ           (1250000u/78125u)       //Time count= (125mS)/(7.8125mS time base).

#define S2_KEY_NO               1                       //S2(PT1.2)  key no.
#define S3_KEY_NO               2                       //S3(PT1.1)  key no.
#define NULL_KEY_NO             0xFF                    //null key no.
#define KEY_DB_TIME             (640000u/78125u)        //Time count= (64mS)/(7.8125mS time base).

#define RTC_TIME_MODE           0x10
#define CALENDAR_MODE           0x20
#define ALARM_TIME_MODE         0x40
#define STOP_WATCH_MODE         0x80

/*----------------------------------------------------------------------------*/
/* Function prototype declaration                                             */
/*----------------------------------------------------------------------------*/
extern void User_Sample_Init(void);
extern void Enter_IDLE_Mode(void);
extern void Release_IDLE_Mode(void);
extern void RTC_Time_Update(void);
extern void Key_Scan_Process(void);
extern void Key_In_Process(void);
extern void Beep_Tone_Process(void);
extern void On_Mode_Display(void);

#endif  //_USER_DEF_H_
