/*******************************************************************************
 *
 * Copyright (c) 2016-2026 HYCON Technology, Inc.
 * All rights reserved.
 * HYCON Technology <www.hycontek.com>
 *
 * HYCON reserves the right to amend this code without notice at any time.
 * HYCON assumes no responsibility for any errors appeared in the code,
 * and HYCON disclaims any express or implied warranty, relating to sale
 * and/or use of this code including liability or warranties relating
 * to fitness for a particular purpose, or infringement of any patent,
 * copyright or other intellectual property right.
 *
 * -----------------------------------------------------------------------------
 * File Name    : main.c
 * IDE tooling  : AndeSight C/C++ IDE, version: 2.0.1 Build ID : 201303201736
 * Device Ver.  : V0.9 crt0.o for HY16F19xx & HY16F18xx MCU.
 * Library Ver. : 1.1
 * MCU Device   : HY16F198B-L100 @3.0V
 * Description  : This file implements the customer's function.
 * Created Date : 2016/06/29
 * Created by   :
 *
 ******************************************************************************/
/*----------------------------------------------------------------------------*/
/* Header file include                                                        */
/*----------------------------------------------------------------------------*/
#include "HY16F198.h"
#include "System.h"
#include "ModuleID.h"
#include "SpecialMacro.h"
#include "Sysinfra.h"
#include "DrvClock.h"
#include "DrvGPIO.h"
#include "DrvLCD.h"
#include "DrvREG32.h"
#include "DrvADC.h"
#include "DrvTimer.h"
#include "DrvUART.h"
#include "user_DEF.h"
#include "DrvRTC.h"
//#include "Display.h"

/*----------------------------------------------------------------------------*/
/* STRUCTURES                                                                 */
/*----------------------------------------------------------------------------*/
#if (0)
volatile typedef union _MCUSTATUS
{
  char  _byte;
  struct
  {
    unsigned b_ADCdone:1;
    unsigned b_TMAdone:1;
    unsigned b_TMBdone:1;
    unsigned b_TMC0done:1;
    unsigned b_TMC1done:1;
    unsigned b_RTCdone:1;
    unsigned b_UART_TxDone:1;
    unsigned b_UART_RxDone:1;
  };
} MCUSTATUS;
MCUSTATUS  MCUSTATUSbits;
#endif

/*----------------------------------------------------------------------------*/
/* Global CONSTANTS                                                           */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* Constant value definition                                                  */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* Function prototype declaration                                             */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* MAIN  function                                                             */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* Function PROTOTYPES                                                        */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* MAIN  function                                                             */
/*----------------------------------------------------------------------------*/
int main(void)
{
    User_Sample_Init();

//------Enter Main program loop-----------------
    while(1)
    {
        //SYS enter IDLE mode !
        Enter_IDLE_Mode();
        SYS_LowPower(1);                        //ݾҦ]Idle  mode^
        Release_IDLE_Mode();

        //SYS wakeup going !
        RTC_Time_Update();
        Key_Scan_Process();
        Key_In_Process();                       //Key in processing.
        Beep_Tone_Process();
        On_Mode_Display();
    }
    return 0;
}

/*---------------------------------------------------------------------------*/
/* Exception Service Routines                                                */
/*---------------------------------------------------------------------------*/
void tlb_exception_handler()
{
    //procedure define by customer.
    asm("nop");
    asm("nop");
}


/******************************************************************************/
/* File Name    : user_ISR.c                                                  */
/* Description  : This file implements the customer's ISR function.           */
/* Created Date : 2016/05/28                                                  */
/* Created by   :                                                             */
/******************************************************************************/
#if (0)
/*----------------------------------------------------------------------------*/
/* Function Name: HW0_ISR()                                                   */
/* Description  : I2C/UART/SPI interrupt Service Routine (HW0).               */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW0_ISR(void)
{

}
#endif

#if (1)
/*----------------------------------------------------------------------------*/
/* Function Name: HW1_ISR()                                                   */
/* Description  : WDT & RTC & Timer A/B/C interrupt Service Routine (HW1).    */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW1_ISR(void)
{
    if( DrvRTC_ReadState() & 0x05 )
    {
        //TAF == 1 ?
        if( DrvRTC_ReadState() & 0x01 )
            UserFlag0.b_ALMInt_Flag = TRUE;
        //PTF == 1 ?
        if( DrvRTC_ReadState() & 0x04 )
        {
            UserFlag0.b_RTCInt_Flag = TRUE;
            if ( UserFlag1.b_StpWGo_Flag == TRUE )
                STW_100 += 78125;                       //Stop Watch 1/100Sec time base ++ ( 1 count= 1/128Sec ).
        }
        DrvRTC_ClearState( E_DRVRTC_CLEAR_ALL );
        DrvRTC_ClearIntFlag();
    }
}
#endif

#if (0)
/*----------------------------------------------------------------------------*/
/* Function Name: HW2_ISR()                                                   */
/* Description  : ADC interrupt Service Routine (HW2).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW2_ISR(void)
{
    if( DrvADC_ReadIntFlag() )
    {
        DrvADC_ClearIntFlag();                          // clear ADC interrupt flag
    }
}
#endif

#if (0)
/*----------------------------------------------------------------------------*/
/* Function Name: HW3_ISR()                                                   */
/* Description  : CMP & OPA interrupt Service Routine (HW3).                  */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW3_ISR(void)
{
    DrvCMP_ClearIntFlag();                      //Clear CMP interrupt flag
}
#endif

#if (0)
/*----------------------------------------------------------------------------*/
/* Function Name: HW4_ISR()                                                   */
/* Description  : PT1 interrupt Service Routine (HW4).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW4_ISR(void)
{
    if ( DrvGPIO_PT1_GetIntFlag() )
    {
        DrvGPIO_PT1_ClearIntFlag( 0x06 );                   //clear PT1.1 & PT1.2 int pending flag.
        UserFlag1.b_KeyInt_Flag = TRUE;
    }
}
#endif

#if (0)
/*----------------------------------------------------------------------------*/
/* Function Name: HW5_ISR()                                                   */
/* Description  : PT2 interrupt Service Routine (HW5).                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW5_ISR(void)
{

}
#endif

#if (0)
/*----------------------------------------------------------------------------*/
/* Function Name: HW7_ISR()                                                   */
/* Description  : UART2 interrupt Service Routine (HW7).                      */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW7_ISR(void)
{

}
#endif

#if (0)
/*----------------------------------------------------------------------------*/
/* Function Name: HW8_ISR()                                                   */
/* Description  : Timer B2 interrupt Service Routine (HW8).                   */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void HW8_ISR(void)
{
    if( DrvTIMER_GetIntFlag (E_TMB2) )
    {
        DrvTIMER_ClearIntFlag(E_TMB2);          // Clear TMB interrupt flag
    }
}
#endif

/*----------------------------------------------------------------------------*/
/* End Of File                                                                */
/*----------------------------------------------------------------------------*/
