/*******************************************************************************
 *
 * Copyright (c) 2016-2026 HYCON Technology, Inc.
 * All rights reserved.
 * HYCON Technology <www.hycontek.com>
 *
 * HYCON reserves the right to amend this code without notice at any time.
 * HYCON assumes no responsibility for any errors appeared in the code,
 * and HYCON disclaims any express or implied warranty, relating to sale
 * and/or use of this code including liability or warranties relating
 * to fitness for a particular purpose, or infringement of any patent,
 * copyright or other intellectual property right.
 *
 * -----------------------------------------------------------------------------
 * File Name    : user_sample.c
 * IDE tooling  : AndeSight C/C++ IDE, version: 2.0.1 Build ID : 201303201736
 * Device Ver.  : V0.9 crt0.o for HY16F19xx & HY16F18xx MCU.
 * Library Ver. : 1.1
 * MCU Device   : HY16F198B-L100 @3.0V
 * Description  : This file implements the customer's function.
 * Created Date : 2016/06/29
 *
 ******************************************************************************/

/*----------------------------------------------------------------------------*/
/* Header file include                                                        */
/*----------------------------------------------------------------------------*/
//#include "stdint.h"
#include "HY16F198.h"
#include "System.h"
//#include "DrvADC.h"
#include "DrvClock.h"
#include "DrvPMU.h"
#include "DrvGPIO.h"
#include "DrvLCD.h"
#include "DrvREG32.h"
#include "DrvRTC.h"
#include "DrvTimer.h"
#include "DrvUART.h"
#include "DrvFlash.h"
#include "ModuleID.h"
#include "SpecialMacro.h"
#include "Sysinfra.h"
#include "user_DEF.h"
#include "user_LCD.h"
#include "DrvRTC.h"

/*----------------------------------------------------------------------------*/
/* Structure definition                                                       */
/*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/* Global variable definition                                                 */
/*----------------------------------------------------------------------------*/
S_DRVRTC_TIME_DATA_T sCurTime;                  //Declare a RTC structure for current RTC time.
S_DRVRTC_TIME_DATA_T sCurAlarm;                 //Declare a RTC structure for current Alarm time.
unsigned char DisplayBuffer[18];
unsigned char Beep_Tone_Cnt;
unsigned char Key_Code_Buf;
unsigned char Key_Code_No;
unsigned char Key_DB_Cnt;
unsigned char Mode_Index;
unsigned int  SetItem_Index;
unsigned int  STW_100, STW_Sec, STW_Min;
unsigned int  Tick500mS_Cnt;
unsigned int  AlmTone_Cnt;
unsigned int  SetMode_Cnt;
unsigned int  QAdjTime_Cnt;


/*----------------------------------------------------------------------------*/
/* Constant value definition                                                  */
/*----------------------------------------------------------------------------*/
const unsigned char code_seg[] =
{
    seg_a+seg_b+seg_c+seg_d+seg_e+seg_f,        //00, char "0"
    seg_b+seg_c,                                //01, char "1"
    seg_a+seg_b+seg_d+seg_e+seg_g,              //02, char "2"
    seg_a+seg_b+seg_c+seg_d+seg_g,              //03, char "3"
    seg_b+seg_c+seg_f+seg_g,                    //04, char "4"
    seg_a+seg_c+seg_d+seg_f+seg_g,              //05, char "5"
    seg_a+seg_c+seg_d+seg_e+seg_f+seg_g,        //06, char "6"
    seg_a+seg_b+seg_c,                          //07, char "7"
    seg_a+seg_b+seg_c+seg_d+seg_e+seg_f+seg_g,  //08, char "8"
    seg_a+seg_b+seg_c+seg_d+seg_f+seg_g,        //09, char "9"
    seg_a+seg_b+seg_c+seg_e+seg_f+seg_g,        //0A, char "A"
    seg_c+seg_d+seg_e+seg_f+seg_g,              //0B, char "b"
    seg_a+seg_d+seg_e+seg_f,                    //0C, char "C"
    seg_b+seg_c+seg_d+seg_e+seg_g,              //0D, char "d"
    seg_a+seg_d+seg_e+seg_f+seg_g,              //0E, char "E"
    seg_a+seg_e+seg_f+seg_g                     //0F, char "F"
};

/*----------------------------------------------------------------------------*/
/* Function prototype declaration                                             */
/*----------------------------------------------------------------------------*/

//------MISC Function declaration---------------
void User_Sample_Init(void);
void Delay(unsigned int num);
void Beep_Tone_Start(void);
void Beep_Tone_Stop(void);
void Key_Scan_Process(void);
void Key_In_Process(void);
void Beep_Tone_Process(void);
void Enter_IDLE_Mode(void);
void Release_IDLE_Mode(void);

//------RTC Function declaration----------------
void RTC_Init(void);
void RTC_Time_Update(void);
void Quick_Adjust_Process(void);
int ComputeWeek(int TempYear, int TempMonth, int TempDay);

//------GPIO Function declaration---------------
void GPIO_Init(void);

//------TIMER Function declaration--------------
void TIMER_Init(void);

//------LCD Function declaration----------------
void LCD_Init(void);
void LCD_RAM_Clear(void);
void LCD_Display_HYcon(void);
void On_Mode_Display(void);
void RAM2LCD( unsigned char *Buffer_Adr );

/*----------------------------------------------------------------------------*/
/* Function Name: User_Sample_Init                                            */
/* Description  : Sample code Initialization Subroutines                      */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void User_Sample_Init(void)
{
    unsigned int InitGo_cnt;
    unsigned char step;

    SYS_DisableGIE();                           //Disable Global Interrupt.

//------Sys. clock setting----------------------
    DrvCLOCK_SelectIHOSC( TRIM_HAO2MHZ );       //Select HAO= 2MHz.
    DrvCLOCK_EnableHighOSC(E_INTERNAL, 60);     //Enable internal HAO.
    DrvCLOCK_EnableLowOSC(E_EXTERNAL, 130000);  //Enable external LSXT 32768Hz.
    DrvCLOCK_SelectMCUClock(0, 0);              //Select MCUCKS= HS_CK/1= 2MHz.
//  DrvCLOCK_SelectMCUClock(1, 0);              //Select MCUCKS= LS_CK/1= 32768Hz.

//------MISC. system initial--------------------
    GPIO_Init();
    LCD_Init();
    RTC_Init();
    TIMER_Init();

    UserFlag0._byte = 0;
    UserFlag1._byte = 0;
    Mode_Index = RTC_TIME_MODE;                 //power on's default mdoe is RTC time mode.
    SYS_EnableGIE( 4, 0x02 );                   //Enable HW1 only.

//------Power on initial going------------------
//Step1: Beep tone & turn all LCD display on about 1Sec.
//Step2: LCD display "HYcon" about 1Sec.
//Step3: Normally system display.
//
    for ( step=1; step <3; step++ )
    {
        if ( step == 1 )
        {
            DrvLCD_DisplayMode( E_LCD_PIXELON );        //all LCD display on.
            Beep_Tone_Start();
        }
        if ( step == 2 )
        {
            DrvLCD_DisplayMode( E_LCD_NORMAL );         //LCD at normal mode.
            Beep_Tone_Stop();
            LCD_Display_HYcon();
        }

        for ( InitGo_cnt= 0; InitGo_cnt < INIT_GO_TIME; )
        {
            if ( UserFlag0.b_RTCInt_Flag == TRUE )
            {
                InitGo_cnt++;
                UserFlag0.b_RTCInt_Flag = FALSE;
            }
        }
    }
    LCD_RAM_Clear();
    UserFlag0.b_DspGo_Flag = TRUE;
    On_Mode_Display();
}

/*----------------------------------------------------------------------------*/
/* Function Name: Key_Scan_Process                                            */
/* Description  : S2 & S3 key scanning, scan period is 7.8125mS.              */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       : PT1.2 & PT1.1 as input with pull high.                      */
/*                 --------------------                                       */
/*                |  PT1.2  |  PT1.1  |                                       */
/*                |   S2    |   S3    |                                       */
/*                |  [00]   |  [04]   |                                       */
/*                 --------------------                                       */
/*                     |         |                                            */
/*                     --------------------- VSS                              */
/*                                                                            */
/*----------------------------------------------------------------------------*/
void Key_Scan_Process(void)
{
    unsigned char key_code;

    if ( UserFlag1.b_KScan_Flag == FALSE )
        return;

//------Key scanning go !-----------------------
    key_code = NULL_KEY_NO;

    if ( DrvGPIO_GetBit( E_PT1, 2 ) == 0 )              // Check S2
    {
            key_code = S2_KEY_NO;
    }
    if ( DrvGPIO_GetBit( E_PT1, 1 ) == 0 )              // Check S3
    {
            key_code = S3_KEY_NO;
    }

//------Key in debounce-------------------------
    Key_DB_Cnt++;
    if ( key_code != Key_Code_Buf )
    {
        Key_Code_Buf = key_code;
        Key_DB_Cnt = 0;
    }
    else
    {
        if (Key_DB_Cnt >= KEY_DB_TIME)
        {
            Key_DB_Cnt = 0;
            if ( Key_Code_Buf == NULL_KEY_NO )
            {
                UserFlag1.b_KeyOn_Flag = FALSE;
                UserFlag1.b_KScan_Flag = FALSE;
                QAdjTime_Cnt = 0;
            }
            else
            {
                if (UserFlag1.b_KeyOn_Flag == FALSE)
                {
                    UserFlag1.b_KeyOn_Flag = TRUE;
                    UserFlag0.b_KInPro_Flag = TRUE;
                    Key_Code_No = Key_Code_Buf;
                }
            }
        }
    }
}

/*----------------------------------------------------------------------------*/
/* Function Name: Key_In_Process                                              */
/* Description  : Key in tasks processing.                                    */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       : key in scan period is 7.8125mS.                             */
/*----------------------------------------------------------------------------*/
void Key_In_Process(void)
{

    //Key processing for 1st key in
    if( UserFlag0.b_KInPro_Flag == TRUE )
    {
        UserFlag0.b_KInPro_Flag = FALSE;
        UserFlag0.b_DspGo_Flag = TRUE;
        SetMode_Cnt = 0;

        //Alarm Tone is going ?
        if ( UserFlag1.b_AlmGo_Flag == TRUE )
        {
            UserFlag1.b_AlmGo_Flag = FALSE;
            Beep_Tone_Start();
            return;
        }

        switch( Key_Code_No )
        {
            //S2 key go !
            case S2_KEY_NO:
            {
                //on View mode !
                if ( UserFlag1.b_SetEI_Flag == FALSE )
                {
                    if ( ( Mode_Index == STOP_WATCH_MODE ) && ( (STW_Min+STW_Sec+STW_100) > 0 ) )
                    {
                        if ( UserFlag1.b_StpWGo_Flag == FALSE )
                        {
                            STW_Min = 0;
                            STW_Sec = 0;
                            STW_100 = 0;
                        }
                    }
                    //to next mode
                    else
                    {
                        Mode_Index = Mode_Index << 1;
                        if ( Mode_Index == 0 ) { Mode_Index = RTC_TIME_MODE; }
                    }
                }
                //on setup mode.
                else
                {
                    SetItem_Index++;
                    QAdjTime_Cnt = KEY_SET_OFF_TIME;
                    if ( SetItem_Index >= 3 ) { SetItem_Index = 0; }
                }
                break;
            }
            //S3 key go !
            case S3_KEY_NO:
            {
                //enter setup mode.
                if ( UserFlag1.b_SetEI_Flag == FALSE )
                {
                    if ( Mode_Index == STOP_WATCH_MODE )
                    {
                        UserFlag1.b_StpWGo_Flag ^= 1;
                    }
                    else
                    {
                        UserFlag1.b_SetEI_Flag = TRUE;
                        SetItem_Index = 0;
                    }
                }
                //current adust go !
                else
                {
                    QAdjTime_Cnt = QADJ_TIME_1HZ;
                    Quick_Adjust_Process();
                }
                break;
            }
            default:
                break;
        }
        Beep_Tone_Start();
    }
    //Key processing for repeat key
    else if ( (UserFlag1.b_KeyOn_Flag == TRUE) && (UserFlag1.b_SetEI_Flag == TRUE) && (QAdjTime_Cnt > 0) )
    {
        SetMode_Cnt = 0;
        QAdjTime_Cnt--;
        if ( QAdjTime_Cnt > 0 )
            return;
        UserFlag0.b_DspGo_Flag = TRUE;

        switch( Key_Code_No )
        {
            //S2 key repeated going
            case S2_KEY_NO:
            {
                UserFlag1.b_SetEI_Flag = FALSE;
                Beep_Tone_Start();
                break;
            }
            //S3 key repeated going
            case S3_KEY_NO:
            {
                QAdjTime_Cnt = QADJ_TIME_8HZ;
                Quick_Adjust_Process();
                break;
            }
            default:
                break;
        }
    }
}

/*----------------------------------------------------------------------------*/
/* Function Name: Enter_IDLE_Mode                                             */
/* Description  : Prepare to enter IDLE mode.                                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void Enter_IDLE_Mode(void)
{
    //Stop HAO ?
    if ( ( Beep_Tone_Cnt == 0 ) && ( UserFlag1._byte == 0 ) )
    {
        Tick500mS_Cnt = 0;
        UserFlag0.b_IRQ128_Flag = FALSE;
        UserFlag0.b_HAO_EI_Flag = FALSE;
        DrvRTC_PeriodicTimeEnable( E_DRVRTC_1_SEC );        //Set RTC int. period time= 1/1Sec= 1Sec.
        DrvCLOCK_SelectMCUClock(1, 0);                      //Select MCUCKS= LS_CK/1= 32768Hz.
        Delay( 5 );
        DrvCLOCK_CloseIHOSC();                              //Stop 2MHz HAO.
        Delay( 5 );
    }
    else
    {
        UserFlag0.b_HAO_EI_Flag = TRUE;
        UserFlag0.b_IRQ128_Flag = TRUE;
        DrvRTC_PeriodicTimeEnable( E_DRVRTC_1_128_SEC );    //Set RTC int. period time= 1/128Sec= 7.8125mS.
    }
}

/*----------------------------------------------------------------------------*/
/* Function Name: Release_IDLE_Mode                                           */
/* Description  : Prepare to release from IDLE mode.                          */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void Release_IDLE_Mode(void)
{
    //Release by KEY ?
    if ( DrvGPIO_PT1_GetIntFlag() )
    {
        DrvGPIO_PT1_ClearIntFlag( 0x06 );       //clear PT1.1 & PT1.2 int pending flag.
        if ( UserFlag1.b_KScan_Flag == FALSE )
            UserFlag0.b_HAO_EI_Flag = FALSE;
        UserFlag1.b_KScan_Flag = TRUE;
    }
    //Enable HAO
    if ( UserFlag0.b_HAO_EI_Flag == FALSE )
    {
        clk_00 = 0x4545;                        //Enable internal 2MHz HAO & enter 32768Hz.
        Delay( 10 );                            //delay 1.5mS for HAO stability when MCUCK= 32768Hz.
        DrvCLOCK_SelectMCUClock(0, 0);          //Select MCUCKS= HS_CK/1= 2MHz.
        Delay( 10 );                            //delay 1.5mS for HAO stability when MCUCK= 32768Hz.
    }
}

/*----------------------------------------------------------------------------*/
/* Function Name: Beep_Tone_Process                                           */
/* Description  : Beep tone time processing.                                  */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void Beep_Tone_Process(void)
{
    if (Beep_Tone_Cnt != 0)
    {
        Beep_Tone_Cnt--;
        if (Beep_Tone_Cnt <= 0)
            Beep_Tone_Stop();
    }
}

/*----------------------------------------------------------------------------*/
/* Function Name: Beep_Tone_Start                                             */
/* Description  : Start PWM beep tone.                                        */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void Beep_Tone_Start(void)
{
    Beep_Tone_Cnt = BEEP_TONE_TIME;
    DrvTMB_Open( E_TMB_MODE0, E_TMB_NORMAL, PWM_4KHZ_TIME );        //set TMB working on normal mode & TMB clock int period= PWM_4KHZ_TIME/HS_CK= 250uS.
    DrvPWM0_Open( 0, 1, 2 );                                        //Set PT2.0= PWMO0, PT2.1= PWMO1 and PWMO0 working on PWMA mdoe with normal pulse output.
    DrvPWM1_Open( 0, 0, 2 );                                        //Set PT2.0= PWMO0, PT2.1= PWMO1 and PWMO1 working on PWMA mdoe with reverse pulse output.
}

/*----------------------------------------------------------------------------*/
/* Function Name: Beep_Tone_Stop                                              */
/* Description  : Stop PWM beep tone.                                         */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void Beep_Tone_Stop(void)
{
    Beep_Tone_Cnt = 0;
    DrvTMB_Close();
    DrvPWM0_Close();
    DrvPWM1_Close();
    DrvGPIO_ClrPortBits( E_PT2, 0x03 );         //set PT2[1:0] as output low.
}

/*----------------------------------------------------------------------------*/
/* Function Name: Delay()                                                     */
/* Description  : Software delay subroutines.                                 */
/* Arguments    : unsigned int num.                                           */
/* Return Value : None.                                                       */
/* Remark       : num= 10,   delay time= 18u S @4MH HAO, 3V.                  */
/*              : num= 100,  delay time= 40u S @4MH HAO, 3V.                  */
/*              : num= 1000, delay time= 260uS @4MH HAO, 3V.                  */
/*                                                                            */
/*----------------------------------------------------------------------------*/
void Delay(unsigned int num)
{
    for( ; num >0; num-- )
        asm( "NOP" );
}

/******************************************************************************/
/* File Name    : user_GPIO.c                                                 */
/* Description  : This file implements the customer's GPIO function.          */
/* Created Date : 2016/05/28                                                  */
/* Created by   :                                                             */
/******************************************************************************/

/*----------------------------------------------------------------------------*/
/* Function Name: GPIO_Init                                                   */
/* Description  : GPIO Initialization Subroutines                             */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void GPIO_Init(void)
{
    DrvGPIO_ClkGenerator( E_LS_CK, 1 );                         //Set IO sampling clock input source is LS_CK, and IOCLK is LS_CK/1.
    DrvGPIO_PT1_EnablePullHigh( 0x06 );                         //enable PT1.1 & PT1.2 pull high 75K ohm.
    DrvGPIO_PT1_EnableINPUT( 0x06 );                            //set PT1.1 & PT1.2  as input port.
    DrvGPIO_PT1_IntTriggerPorts( 0x06, E_N_Edge );              //set PT1.1 & PT1.2 as interrupt trigger method is negative edge.
    DrvGPIO_PT1_DisableOUTPUT( 0xFF );                          //close Output PT1[7:0] OE.
    DrvGPIO_PT1_EnableINT( 0x06 );                              //PT1.1 & PT1.2 interrupt enable.
    DrvGPIO_PT1_ClearIntFlag( 0xFF );                           //clear PT1[7:0] interrupt flag.

//------PWM function initial for Buzzer---------
    DrvGPIO_Open( E_PT2, 0x03, E_IO_OUTPUT );                   //set PT2[1:0] as output port.
    DrvPWM_CountCondition( PWM_4KHZ_TIME/2, PWM_4KHZ_TIME/2 );  //Set TBC1 as 50% duty, TBC2 as 50% duty.
    DrvGPIO_ClrPortBits( E_PT2, 0x03 );                         //set PT2[1:0] as output low.
}


/******************************************************************************/
/* File Name    : user_RTC.c                                                  */
/* Description  : This file implements the customer's RTC function.           */
/* Created Date : 2016/05/28                                                  */
/* Created by   :                                                             */
/******************************************************************************/
/*----------------------------------------------------------------------------*/
/* Function Name: RTC_Init                                                    */
/* Description  : RTC Initialization Subroutines                              */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void RTC_Init(void)
{
    DrvRTC_ClkConfig( 1 );                              //Enable RTC clock, ENRTCK = 1.
    DrvRTC_WriteEnable();                               //Unlock RTC IP writing function.
    DrvRTC_ClockSource( 0 );                            //Select RTC's clock source from 32768Hz LSXT.
    DrvRTC_Enable();                                    //Enable RTC IP.

//------initial current RTC time----------------
//default RTC time is 00:00 24Hr Jan/01/2016.
//
    sCurTime.u8cClockDisplay = E_DRVRTC_HOUR_24;        //set RTC Hour format is 24H.
    sCurTime.u8cAmPm = 0;                               //set RTC time is AM.
    sCurTime.u32cSecond = 0;
    sCurTime.u32cMinute = 0;
    sCurTime.u32cHour = 0;
    sCurTime.u32cDay =  1;
    sCurTime.u32cMonth = 1;
    sCurTime.u32Year = 2016;
    sCurTime.u32cDayOfWeek = ComputeWeek(sCurTime.u32Year, sCurTime.u32cMonth, sCurTime.u32cDay);
    DrvRTC_Write( DRVRTC_CURRENT_TIME, &sCurTime);

//------initial current Alarm time--------------
//default Alarm Time is 00:00 24Hr Jan/01/2016.
//
    sCurAlarm.u8cClockDisplay = E_DRVRTC_HOUR_24;       //set alarm Hour format is 24H.
    sCurAlarm.u8cAmPm = 0;                              //set alarm time is AM.
    sCurAlarm.u32cSecond = 0;
    sCurAlarm.u32cMinute = 0;
    sCurAlarm.u32cHour = 0;
    sCurAlarm.u32cDay = sCurTime.u32cDay;
    sCurAlarm.u32cMonth = sCurTime.u32cMonth;
    sCurAlarm.u32Year = sCurTime.u32Year = 2016;
    sCurAlarm.u32cDayOfWeek = sCurTime.u32cDayOfWeek;
    DrvRTC_Write( DRVRTC_ALARM_TIME, &sCurTime);

//------RTC time running !----------------------
    DrvRTC_PeriodicTimeEnable(E_DRVRTC_1_128_SEC);      //Set RTC int. period time= 1/128Sec= 7.8125mS.
    DrvRTC_ClearIntFlag();                              //Clear RTC int. request flag.
    DrvRTC_EnableInt();                                 //Enable RTC interrupt !
}

/*----------------------------------------------------------------------------*/
/* Function Name: RTC_Time_Update                                             */
/* Description  : RTC display                                                 */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void RTC_Time_Update(void)
{

//------ALARM compare with TIME-----------------
    UserFlag0.b_ALMInt_Flag = FALSE;
    DrvRTC_Read( DRVRTC_CURRENT_TIME, &sCurTime );
    if ( (UserFlag0.b_AlmEI_Flag == TRUE)
    && (sCurTime.u32cHour == sCurAlarm.u32cHour)
    && (sCurTime.u32cMinute == sCurAlarm.u32cMinute)
    && (sCurTime.u32cSecond == sCurAlarm.u32cSecond) )
    {
        UserFlag1.b_AlmGo_Flag = TRUE;
        AlmTone_Cnt = 0;
    }

//------RTC IRQ processing----------------------
    if ( UserFlag0.b_RTCInt_Flag == FALSE )
        return;
    UserFlag0.b_RTCInt_Flag = FALSE;

    //Tick time update !
    if ( UserFlag0.b_IRQ128_Flag == TRUE )
    {
        Tick500mS_Cnt++;
        if ( Tick500mS_Cnt > (10000000u/78125u) ) { Tick500mS_Cnt = 0; }
    }

    //Alarm Tone process !
    if ( UserFlag1.b_AlmGo_Flag == TRUE )
    {
        //halt tick process !
        if ( Tick500mS_Cnt < ( 5000000u/78125u) )
        {
            if ( (Tick500mS_Cnt & 0x0F) == 0x08 )
            {
                AlmTone_Cnt++;
                if ( AlmTone_Cnt >= ALM_TONE_TIME )
                {
                    UserFlag1.b_AlmGo_Flag = FALSE;
                    Beep_Tone_Stop();
                }
                else
                    Beep_Tone_Start();
            }
            else if ( (Tick500mS_Cnt & 0x0F) == 0 )
                Beep_Tone_Stop();
        }
        UserFlag0.b_DspGo_Flag = TRUE;
    }

    //Stop Watch time update !
    if ( UserFlag1.b_StpWGo_Flag == TRUE )
    {
        //update if over 1Sec.
        if ( STW_100 >= (128*78125) )
        {
            STW_100 = 0;
            STW_Sec++;
            if ( STW_Sec >= 60 )
            {
                STW_Sec = 0;
                STW_Min++;
                if ( STW_Min >= 100 ) { STW_Min = 0; }
            }
        }
        UserFlag0.b_DspGo_Flag = TRUE;
    }

    //Setup mode time counting !
    if ( UserFlag1.b_SetEI_Flag == TRUE )
    {
        SetMode_Cnt++;
        if ( SetMode_Cnt >= SET_RELEASE_TIME )
            UserFlag1.b_SetEI_Flag = FALSE;
        UserFlag0.b_DspGo_Flag = TRUE;
    }

    //don't update display if mode is ALARM and no any events.
    if ( !((Mode_Index == ALARM_TIME_MODE) && (Beep_Tone_Cnt == 0) && (UserFlag1._byte == 0)) )
        UserFlag0.b_DspGo_Flag = TRUE;
}

/*----------------------------------------------------------------------------*/
/* Function Name: Quick_Adjust_Process                                        */
/* Description  : Current time quickly adjust increase.                       */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void Quick_Adjust_Process(void)
{
//------Adjust go for Alarm set mode------------
    if ( Mode_Index == ALARM_TIME_MODE )
    {
        DrvRTC_Read( DRVRTC_ALARM_TIME, &sCurTime);

        if ( SetItem_Index == 0 )
        {
            sCurAlarm.u32cHour++;
            if ( sCurAlarm.u32cHour >= 24 )
                sCurAlarm.u32cHour = 0;
        }
        if ( SetItem_Index == 1 )
        {
            sCurAlarm.u32cMinute++;
            if ( sCurAlarm.u32cMinute >= 60 )
                sCurAlarm.u32cMinute = 0;
        }
        if ( SetItem_Index == 2 )
        {
            QAdjTime_Cnt = 0;
            if ( UserFlag0.b_AlmEI_Flag == TRUE )
            {
              //DrvRTC_AlarmDisable();
                UserFlag0.b_AlmEI_Flag = FALSE;
                UserFlag0.b_ALMInt_Flag = FALSE;
            }
            else
            {
                UserFlag0.b_AlmEI_Flag = TRUE;
              //DrvRTC_AlarmEnable();
            }
        }
        DrvRTC_Write( DRVRTC_ALARM_TIME, &sCurTime);
    }

//------Adjust go for RTC set mode--------------
    if ( (Mode_Index == RTC_TIME_MODE) || (Mode_Index == CALENDAR_MODE) )
    {
        DrvRTC_Read( DRVRTC_CURRENT_TIME, &sCurTime );

        //current time adjust
        if ( Mode_Index == RTC_TIME_MODE )
        {
            if ( SetItem_Index == 0 )
            {
                sCurTime.u32cHour++;
                if ( sCurTime.u32cHour >= 24 )
                    sCurTime.u32cHour = 0;
            }
            if ( SetItem_Index == 1 )
            {
                sCurTime.u32cMinute++;
                if ( sCurTime.u32cMinute >= 60 )
                    sCurTime.u32cMinute = 0;
            }
            if ( SetItem_Index == 2 )
            {
                QAdjTime_Cnt = 0;
                sCurTime.u32cSecond = 0;
            }
            DrvRTC_Write( DRVRTC_CURRENT_TIME, &sCurTime );
        }
        //calendar adjust
        else
        {
            if ( SetItem_Index == 0 )
            {
                sCurTime.u32Year++;
                if ( sCurTime.u32Year >= 2100 )
                    sCurTime.u32Year = 2016;
            }
            if ( SetItem_Index == 1 )
            {
                sCurTime.u32cMonth++;
                if ( sCurTime.u32cMonth >= 13 )
                    sCurTime.u32cMonth = 1;
            }
            if ( SetItem_Index == 2 )
            {
                sCurTime.u32cDay++;
                if ( (sCurTime.u32cMonth == 2) && (sCurTime.u32cDay >= 30) )
                    sCurTime.u32cDay = 1;
                else if ( (sCurTime.u32cDay >= 31) && ((sCurTime.u32cMonth == 4) || (sCurTime.u32cMonth == 6) || (sCurTime.u32cMonth == 9) || (sCurTime.u32cMonth == 11)) )
                    sCurTime.u32cDay = 1;
                else if ( sCurTime.u32cDay >= 32 )
                    sCurTime.u32cDay = 1;
            }
            DrvRTC_Write( DRVRTC_CURRENT_TIME, &sCurTime );

            //Leap year checking
            DrvRTC_Read( DRVRTC_CURRENT_TIME, &sCurTime );
            if ( ((rtc_00 & 0x00080000) == 0) && (sCurTime.u32cMonth == 2) && (sCurTime.u32cDay > 28) )
                sCurTime.u32cDay = 1;
            DrvRTC_Write( DRVRTC_CURRENT_TIME, &sCurTime );

            sCurTime.u32cDayOfWeek = ComputeWeek(sCurTime.u32Year, sCurTime.u32cMonth, sCurTime.u32cDay);
        }
    }
}

/*----------------------------------------------------------------------------*/
/* Function Name: ComputeWeek                                                 */
/* Description  : Compute Week Subroutines.                                   */
/* Arguments    : int TempYear, int TempMonth, int TempDay                    */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
int ComputeWeek(int TempYear, int TempMonth, int TempDay)
{
    int TempWeek;
    if (TempMonth >= 3)
    {
        TempMonth = TempMonth - 2;
    }
    else
    {
        TempMonth = TempMonth + 10;
        TempYear--;
    }

    TempWeek = TempYear + (int)(TempYear / 4) - (int)(TempYear / 100) + (int)(TempYear / 400) + (int)(2.6 * TempMonth - 0.2) + TempDay;
    TempWeek = TempWeek - 7*(int)(TempWeek / 7);
    return (TempWeek);
}


/******************************************************************************/
/* File Name    : user_TIMER.c                                                */
/* Description  : This file implements the customer's TIMER unction.          */
/* Created Date : 2016/05/28                                                  */
/* Created by   :                                                             */
/******************************************************************************/

/*----------------------------------------------------------------------------*/
/* Function Name: TIMER_Init                                                  */
/* Description  : TIMER Initialization Subroutines.                           */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       : Current HS_CK is from 2MHz HAO.                             */
/*----------------------------------------------------------------------------*/
void TIMER_Init(void)
{
//------Initial TIMER B for PWM-----------------
    DrvTMBC_Clk_Source( E_HS_CK, 0 );                           //set TMB Clock from HS_CK/1.
    DrvTMB_Open( E_TMB_MODE0, E_TMB_NORMAL, PWM_4KHZ_TIME );    //set TMB working on normal mode & TMB clock int period= PWM_4KHZ_TIME/HS_CK= 250uS.
}


/******************************************************************************/
/* File Name    : user_LCD.c                                                  */
/* Description  : This file implements the customer's LCD function.           */
/* Created Date : 2016/05/28                                                  */
/* Created by   :                                                             */
/******************************************************************************/

/*----------------------------------------------------------------------------*/
/* Function Name: LCD_Init                                                    */
/* Description  : LCD Initialization Subroutines                              */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       : 1. LCD working@ 4COM x 36SEG, VLCD= from charge pump 2.93V. */
/*              : 2. Charge pump's clock from HS_CK/16= 4M/16= 250KHz).       */
/*              : 3. LCD CLK(LCK) from (HS_CK/64)/9/16= 4M/9216= 434Hz.       */
/*              :    so LCD frame Freq. = 434Hz/(4x2)= 54Hz.                  */
/*----------------------------------------------------------------------------*/
void LCD_Init(void)
{
    clk_10 = 0xFF00FF42;                        //Set LCK= (LS_CK/8)/9/1= (32768/8)/9/1= 455Hz & charge pump clock = LS_CK/1.
    DrvLCD_VLCDMode( E_VLCD27 );
    DrvLCD_VLCDTrim( 3 );                       //Trim VLCD as 2.93V.
    DrvLCD_LcdDuty ( E_LCD_DUTY4 );             //enable LCD@1/4 duty.

    DrvLCD_IOMode( 0, 0xFF );                   //Set PT6(SEG2 ~SEG9 ) as LCD mode.
    DrvLCD_IOMode( 1, 0x7F );                   //Set PT7(SEG10~SEG16) as LCD mode.
    DrvLCD_IOMode( 5, 0xFF );                   //Set SEG1/0 as LCD mode.
    DrvLCD_LCDBuffer( ENABLE );                 //enable LCD working.
    DrvLCD_DisplayMode( E_LCD_NORMAL );         //LCD at normal mode.
    LCD_RAM_Clear();
}

/*----------------------------------------------------------------------------*/
/* Function Name: LCD_RAM_Clear                                               */
/* Description  : LCD display RAM clear.                                      */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void LCD_RAM_Clear(void)
{
    int i=0;
    for(i=0; i< SEG_BUF_SIZE; i++)
    {
        DisplayBuffer[i] = 0;
    }
}

/*----------------------------------------------------------------------------*/
/* Function Name: On_Mode_Display                                             */
/* Description  : Current Mode display updated                                */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       : LCD panel define as below                                   */
/*                                                                            */
/*            -----    -----    -----    -----    -----    -----              */
/*    ////   |     |  |     |  |     |  |     |  |     |  |     |  m  A  >    */
/*           |     |  |     |  |     |  |     |  |     |  |     |  K  X  >    */
/*    ----    -----    -----    -----    -----    -----    -----   M  V  >    */
/*           |     |  |     |  |     |  |     |  |     |  |     |        >    */
/*    TARE   |     |  |     |  |     |  |     |  |     |  |     | 'E  g       */
/*    ZERO    ----- o  ----- o  ----- o  ----- o  ----- o  ----- o            */
/*                                                                            */
/*            Dig5     Dig4]    Dig3     Dig2     Dig1     Dig0               */
/*                                                                            */
/* Note: Dig0's LCD buff nest is Buf5.                                        */
/*       Dig1's LCD buff nest is Buf4.                                        */
/*       Dig2's LCD buff nest is Buf3.                                        */
/*       Dig3's LCD buff nest is Buf2.                                        */
/*       Dig4's LCD buff nest is Buf1.                                        */
/*       Dig5's LCD buff nest is Buf0.                                        */
/*       "////" LCD buff nest is high nibble of Buf6.                         */
/*       "A" LCD buff nest is low nibble of Buf7[1].                          */
/*       ">>>>" LCD buff nest is high nibble of Buf8.                         */
/*----------------------------------------------------------------------------*/
void On_Mode_Display(void)
{
    unsigned char Dig5_buf, Dig4_buf, Dig3_buf, Dig2_buf, Dig1_buf, Dig0_buf;

    if ( UserFlag0.b_DspGo_Flag == FALSE )
        return;
    UserFlag0.b_DspGo_Flag = FALSE;

//------Stop Watch display----------------------
    if ( Mode_Index == STOP_WATCH_MODE )
    {
        Dig5_buf = code_seg[ (STW_Min/10) % 10 ];
        Dig4_buf = code_seg[ (STW_Min) % 10 ];
        Dig3_buf = code_seg[ (STW_Sec/10) % 10 ];
        Dig2_buf = code_seg[ (STW_Sec) % 10 ];
        Dig1_buf = code_seg[ (STW_100/1000000) % 10 ];
        Dig0_buf = code_seg[ (STW_100/100000) % 10 ];
    }

//------Alarm Time display----------------------
    if ( Mode_Index == ALARM_TIME_MODE )
    {
        DrvRTC_Read( DRVRTC_ALARM_TIME, &sCurTime);
        Dig5_buf = 0;
        Dig4_buf = 0;
        Dig3_buf = code_seg[ (sCurAlarm.u32cHour/10) % 10 ];
        Dig2_buf = code_seg[ (sCurAlarm.u32cHour) % 10 ];
        Dig1_buf = code_seg[ (sCurAlarm.u32cMinute/10) % 10 ];
        Dig0_buf = code_seg[ (sCurAlarm.u32cMinute) % 10 ];

        if( UserFlag1.b_SetEI_Flag == TRUE )
        {
            if ( (SetItem_Index == 0) && (Tick500mS_Cnt > (5000000u/78125u)) )
            {
                if ( (QAdjTime_Cnt == 0) || ((QAdjTime_Cnt != 0) && (Key_Code_No == S2_KEY_NO)) )
                {
                    Dig3_buf = 0;
                    Dig2_buf = 0;
                }
            }
            if ( (SetItem_Index == 1)  && (Tick500mS_Cnt > (5000000u/78125u)) )
            {
                if ( (QAdjTime_Cnt == 0) || ((QAdjTime_Cnt != 0) && (Key_Code_No == S2_KEY_NO)) )
                {
                    Dig1_buf = 0;
                    Dig0_buf = 0;
                }
            }
            if ( SetItem_Index == 2 )
            {
                Dig5_buf = Char_A;
                Dig4_buf = Char_L;
                Dig3_buf = 0;
                Dig2_buf = 0;
                Dig1_buf = 0;
                Dig0_buf = 0;
                if ( (Tick500mS_Cnt < (5000000u/78125u)) && (UserFlag0.b_AlmEI_Flag == TRUE) )
                {
                    Dig2_buf = Char_o;
                    Dig1_buf = Char_n;
                }
                else if ( (Tick500mS_Cnt < (5000000u/78125u)) && (UserFlag0.b_AlmEI_Flag == FALSE) )
                {
                    Dig2_buf = Char_o;
                    Dig1_buf = Char_F;
                    Dig0_buf = Char_F;
                }
            }
        }
    }

//------Time & Calendar Display-----------------
    if ( (Mode_Index == RTC_TIME_MODE) || (Mode_Index == CALENDAR_MODE) )
    {
        DrvRTC_Read( DRVRTC_CURRENT_TIME, &sCurTime );

        if ( Mode_Index == RTC_TIME_MODE )
        {
            Dig5_buf = code_seg[ (sCurTime.u32cHour/10) % 10 ];
            Dig4_buf = code_seg[ (sCurTime.u32cHour) % 10 ];
            Dig3_buf = code_seg[ (sCurTime.u32cMinute/10) % 10 ];
            Dig2_buf = code_seg[ (sCurTime.u32cMinute) % 10 ];
            Dig1_buf = code_seg[ (sCurTime.u32cSecond/10) % 10 ];
            Dig0_buf = code_seg[ (sCurTime.u32cSecond) % 10 ];
        }
        else
        {
            Dig5_buf = code_seg[ (sCurTime.u32Year/10) % 10 ];
            Dig4_buf = code_seg[ (sCurTime.u32Year) % 10 ];
            Dig3_buf = code_seg[ (sCurTime.u32cMonth/10) % 10 ];
            Dig2_buf = code_seg[ (sCurTime.u32cMonth) % 10 ];
            Dig1_buf = code_seg[ (sCurTime.u32cDay/10) % 10 ];
            Dig0_buf = code_seg[ (sCurTime.u32cDay) % 10 ];
        }

        if( (UserFlag1.b_SetEI_Flag == TRUE) && (Tick500mS_Cnt > (5000000u/78125u)) )
        {
            if ( (SetItem_Index == 0) && ( (QAdjTime_Cnt == 0) || ((QAdjTime_Cnt != 0) && (Key_Code_No == S2_KEY_NO)) ) )
            {
                Dig5_buf = 0;
                Dig4_buf = 0;
            }
            if ( (SetItem_Index == 1) && ( (QAdjTime_Cnt == 0) || ((QAdjTime_Cnt != 0) && (Key_Code_No == S2_KEY_NO)) ) )
            {
                Dig3_buf = 0;
                Dig2_buf = 0;
            }
            if ( (SetItem_Index == 2) && ( (QAdjTime_Cnt == 0) || ((QAdjTime_Cnt != 0) && (Key_Code_No == S2_KEY_NO)) ) )
            {
                Dig1_buf = 0;
                Dig0_buf = 0;
            }
        }
    }

    //Alarm icon display
    DisplayBuffer[7] = 0;
    if ( UserFlag0.b_AlmEI_Flag == TRUE )
    {
        if ( ! ((UserFlag1.b_AlmGo_Flag == TRUE) && (Tick500mS_Cnt < (5000000u/78125u) )) )
            DisplayBuffer[7] = ALARM_LCD_SIGN;
    }

    DisplayBuffer[0] = Dig5_buf;
    DisplayBuffer[1] = Dig4_buf;
    if ( !(Mode_Index == ALARM_TIME_MODE) )
        DisplayBuffer[1] |= seg_h;                                  //point display.
    DisplayBuffer[2] = Dig3_buf;
    DisplayBuffer[3] = Dig2_buf;
    if ( !( (Mode_Index == ALARM_TIME_MODE) && (UserFlag1.b_SetEI_Flag == TRUE) && (SetItem_Index == 2) ) )
        DisplayBuffer[3] |= seg_h;                                  //point display.
    DisplayBuffer[4] = Dig1_buf;
    DisplayBuffer[5] = Dig0_buf;
    DisplayBuffer[8] = Mode_Index;
    RAM2LCD( DisplayBuffer );                                       //Update LCD display RAM.
}

/*----------------------------------------------------------------------------*/
/* Function Name: LCD_Display_HYcon                                           */
/* Description  : Display "HYcon" char on LCD2 panel.                         */
/* Arguments    : None.                                                       */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void LCD_Display_HYcon(void)
{
    DisplayBuffer[1] = Char_H;
    DisplayBuffer[2] = Char_Y;
    DisplayBuffer[3] = Char_c;
    DisplayBuffer[4] = Char_o;
    DisplayBuffer[5] = Char_n;
    RAM2LCD( DisplayBuffer );                                       //Update LCD display RAM.
}

/*----------------------------------------------------------------------------*/
/* Function Name: RAM2LCD                                                     */
/* Description  : RAM buffer data transfer to LCD RAM                         */
/* Arguments    : unsigned char *Buffer_Adr => buffer pointer address.        */
/* Return Value : None.                                                       */
/* Remark       :                                                             */
/*----------------------------------------------------------------------------*/
void RAM2LCD( unsigned char *Buffer_Adr )
{
    unsigned char buf_idx;

    for(buf_idx=0; buf_idx<SEG_BUF_SIZE; buf_idx++)
    {
        DrvLCD_WriteData ( buf_idx, *Buffer_Adr );
        Buffer_Adr++;
    }
}


/*----------------------------------------------------------------------------*/
/* End Of File                                                                */
/*----------------------------------------------------------------------------*/
